
from kotonebot.tasks.common import sprite_path
from kotonebot.backend.core import Image, HintBox, HintPoint



class Common:
    
    ButtonClose = Image(path=sprite_path(r"1792b27f-b8ab-424f-8c74-307230373823.png"), name="button_close.png")

    ButtonCommuFastforward = Image(path=sprite_path(r"d03227bd-d726-402a-8a16-9639a14c3b7b.png"), name="button_commu_fastforward.png")

    ButtonCompletion = Image(path=sprite_path(r"49eec332-87b5-4e82-b54b-2c0abf98f029.png"), name="button_completion.png")

    ButtonConfirm = Image(path=sprite_path(r"b01539ab-6449-4732-a879-22df632dbbd6.png"), name="button_confirm.png")

    ButtonConfirmNoIcon = Image(path=sprite_path(r"c43e4598-e8b7-4d6c-b41d-9ea0c910fd06.png"), name="button_confirm_no_icon.png")

    ButtonContest = Image(path=sprite_path(r"7ea01c5f-5c01-420b-8e37-36835fa3571c.png"), name="button_contest.png")

    ButtonEnd = Image(path=sprite_path(r"b95a376e-645c-4ba5-991e-6644f9a74b13.png"), name="button_end.png")

    ButtonHome = Image(path=sprite_path(r"47fe7859-20e8-4a46-b92b-8bcf7bb9f2dc.png"), name="button_home.png")

    ButtonIconArrowShort = Image(path=sprite_path(r"f3656fcc-a150-42bc-824d-9d40633ee111.png"), name="button_icon_arrow_short.png")

    ButtonIconArrowShortDisabled = Image(path=sprite_path(r"fdc00ad3-97db-4494-a55b-a7e14fa9a756.png"), name="button_icon_arrow_short_disabled.png")

    ButtonIconCheckMark = Image(path=sprite_path(r"b64b8cca-338e-4fc5-999d-50ad21a4b2da.png"), name="button_icon_check_mark.png")

    ButtonIconClose = Image(path=sprite_path(r"3c7b2002-c3ca-43cc-9008-24d207f34588.png"), name="button_icon_close.png")

    ButtonIdol = Image(path=sprite_path(r"884d10e1-fd5a-4254-823b-256937e9c4a5.png"), name="button_idol.png")

    ButtonIdolSupportCard = Image(path=sprite_path(r"bc06cce3-1e95-4f9f-a583-f28b7c9160e5.png"), name="button_idol_support_card.png")

    ButtonNext = Image(path=sprite_path(r"f4ad0154-d131-44e4-ac27-a22f82e9ed08.png"), name="button_next.png")

    ButtonNextNoIcon = Image(path=sprite_path(r"03098b77-1501-416f-a7f5-f70f73c9f89e.png"), name="button_next_no_icon.png")

    ButtonRetry = Image(path=sprite_path(r"99d6049d-8c2a-45f4-a459-aff5ccba2e49.png"), name="button_retry.png")

    ButtonSelect = Image(path=sprite_path(r"d20ad0e2-6161-4e67-b0fc-fb2aec2877ab.png"), name="button_select.png")

    ButtonStart = Image(path=sprite_path(r"f1a62fb7-382b-44d1-afe7-8026c2f85eed.png"), name="button_start.png")

    ButtonToolbarHome = Image(path=sprite_path(r"94e94dcc-2ae2-44d9-ac15-c6428b1feb0b.png"), name="button_toolbar_home.png")

    ButtonToolbarMenu = Image(path=sprite_path(r"1a9b749e-9994-4070-b837-84969f6e926b.png"), name="button_toolbar_menu.png")

    CheckboxUnchecked = Image(path=sprite_path(r"b03b1726-2680-4010-b2b5-ef786cff3709.png"), name="checkbox_unchecked.png")

    ButtonToolbarHomeBinary = Image(path=sprite_path(r"2f68e457-4491-4ebc-b436-e3e63db92e14.png"), name="底部工具栏 首页按钮 二值化图片")

    ButtonToolbarMenuBinary = Image(path=sprite_path(r"bf6dcf21-2ffc-441f-82f3-59319f2313b1.png"), name="底部工具栏 菜单按钮 二值化图片")

    TextGameUpdate = Image(path=sprite_path(r"140066ba-0926-4ba2-84ab-8e27c321c52a.png"), name="text_game_update.png")

    TextNetworkError = Image(path=sprite_path(r"cc2d7443-704d-4623-9b1c-da3341f3fa2e.png"), name="text_network_error.png")

    ButtonCommuSkip = Image(path=sprite_path(r"f1f21925-3e22-4dd1-b53b-bb52bcf26c2b.png"), name="跳过交流按钮")

    ButtonOK = Image(path=sprite_path(r"8424ecdd-8857-4764-9fd0-d4bfa440c128.png"), name="OK 按钮")

    ButtonSelect2 = Image(path=sprite_path(r"5ebcde3b-f0fd-4e5d-b3de-ada8f0b5e03b.png"), name="選択する")

    IconButtonCheck = Image(path=sprite_path(r"fad5eec2-5fd5-412f-9abb-987a3087dc54.png"), name="按钮✓图标")

    IconButtonCross = Image(path=sprite_path(r"bc7155ac-18c9-4335-9ec2-c8762d37a057.png"), name="按钮×图标")


    pass
class Daily:
    
    ButonLinkData = Image(path=sprite_path(r"45148efb-d0d3-44b1-a3fa-7d828593a84c.png"), name="buton_link_data.png")

    ButtonAssignmentPartial = Image(path=sprite_path(r"f5487e79-114f-4d39-bff5-365f31114e21.png"), name="button_assignment_partial.png")

    ButtonClaimAllNoIcon = Image(path=sprite_path(r"1c88a1ab-5904-435c-928c-6434465b491a.png"), name="button_claim_all_no_icon.png")

    ButtonClubCollectReward = Image(path=sprite_path(r"6d805420-15dc-4b8c-9b9b-eb6f9015db6f.png"), name="button_club_collect_reward.png")

    ButtonClubSendGift = Image(path=sprite_path(r"0340eebc-084e-4566-b7c3-3f175c523614.png"), name="button_club_send_gift.png")

    ButtonClubSendGiftNext = Image(path=sprite_path(r"a0c121d3-6aa1-4e07-9024-fef4e8d48d1f.png"), name="button_club_send_gift_next.png")

    ButtonContestChallenge = Image(path=sprite_path(r"813b659e-25dc-47de-a9fb-5423ca6664da.png"), name="button_contest_challenge.png")

    ButtonContestChallengeStart = Image(path=sprite_path(r"51970f82-1d78-49d8-a099-e381b143dbfb.png"), name="button_contest_challenge_start.png")

    ButtonContestRanking = Image(path=sprite_path(r"3961e7f6-2a2e-4fee-9f86-19efd1d14e62.png"), name="button_contest_ranking.png")

    ButtonContestStart = Image(path=sprite_path(r"e2d8ffdc-f406-4134-8f52-7f06d2e85c23.png"), name="button_contest_start.png")

    ButtonDailyShop = Image(path=sprite_path(r"ab3ce6a4-bd60-412f-a8bc-1107d77a2590.png"), name="button_daily_shop.png")

    ButtonHomeCurrent = Image(path=sprite_path(r"14a6adaa-a791-48eb-b396-588edc7bc65d.png"), name="button_home_current.png")

    ButtonIconPass = Image(path=sprite_path(r"bbc23322-d73f-4884-a9b7-8a773887d262.png"), name="button_icon_pass.png")

    ButtonIconSkip = Image(path=sprite_path(r"d8ae4988-257a-4807-945b-94fd8a72518f.png"), name="button_icon_skip.png")

    ButtonMission = Image(path=sprite_path(r"8ce04389-a253-4cb3-a558-730dc3df5f1c.png"), name="button_mission.png")

    ButtonPass = Image(path=sprite_path(r"9501779e-1aaf-456a-81f2-f68af8344c58.png"), name="button_pass.png")

    ButtonPassClaim = Image(path=sprite_path(r"b9d6d4c8-5bef-4ac5-98c3-5514bea7d68f.png"), name="button_pass_claim.png")

    ButtonPresentsPartial = Image(path=sprite_path(r"d4fe14e7-3cf4-4827-aef3-13db86469014.png"), name="button_presents_partial.png")

    ButtonProduce = Image(path=sprite_path(r"3b142a19-d497-412f-9212-3182f9ba3557.png"), name="button_produce.png")

    ButtonShop = Image(path=sprite_path(r"7d06a260-3873-4325-92c5-0e0760c4b399.png"), name="button_shop.png")

    ButtonShopCapsuleToys = Image(path=sprite_path(r"33b53d6a-05eb-40fb-9b80-11c95e5586d5.png"), name="button_shop_capsule_toys.png")

    ButtonShopCapsuleToysDraw = Image(path=sprite_path(r"1092a47a-73e3-40fd-abe3-8a72056c7b42.png"), name="button_shop_capsule_toys_draw.png")

    ButtonShopCountAdd = Image(path=sprite_path(r"ace18c2f-50bd-4d62-9004-447b0ef2bbb5.png"), name="button_shop_count_add.png")

    ButtonShopCountAddDisabled = Image(path=sprite_path(r"2cd5f2aa-561e-4483-9b1c-7bf9f0f6afdc.png"), name="button_shop_count_add_disabled.png")

    ButtonSupportCardUpgrade = Image(path=sprite_path(r"4624770e-2753-4ff2-ac5a-093d9e1f82df.png"), name="button_support_card_upgrade.png")

    IconTitleDailyShop = Image(path=sprite_path(r"e9ee330d-dfca-440e-8b8c-0a3b4e8c8730.png"), name="日常商店标题图标")

    BoxHomeAssignment = HintBox(x1=16, y1=642, x2=127, y2=752, source_resolution=(720, 1280))

    BoxHomeAP = HintBox(x1=291, y1=4, x2=500, y2=82, source_resolution=(720, 1280))

    BoxHomeJewel = HintBox(x1=500, y1=7, x2=703, y2=82, source_resolution=(720, 1280))

    BoxHomeActivelyFunds = HintBox(x1=11, y1=517, x2=137, y2=637, source_resolution=(720, 1280))

    IconAssignKouchou = Image(path=sprite_path(r"14d3756c-001f-441c-aa81-0f6e74fafcd5.png"), name="icon_assign_kouchou.png")

    IconAssignMiniLive = Image(path=sprite_path(r"92ec7491-a1a7-4b3d-95ea-50852eeedf92.png"), name="icon_assign_mini_live.png")

    IconAssignOnlineLive = Image(path=sprite_path(r"a89e1b07-4f22-4ebb-ae27-1a948cf65350.png"), name="icon_assign_online_live.png")

    IconAssignTitle = Image(path=sprite_path(r"b7437da9-98e8-4261-b3ad-5d6c7c6a7b45.png"), name="icon_assign_title.png")

    IconMenuClub = Image(path=sprite_path(r"1fb58cec-d478-41f8-b0a0-826901322500.png"), name="icon_menu_club.png")

    IconShopAp = Image(path=sprite_path(r"6f38e88d-eb40-4769-898d-6a951ebfef4f.png"), name="icon_shop_ap.png")

    IconShopMoney = Image(path=sprite_path(r"e09005f4-7476-41ea-853d-8903e880f7b2.png"), name="icon_shop_money.png")

    IconShopTitle = Image(path=sprite_path(r"596ed0e2-15c3-4856-a2ba-44e4bb8ebdb9.png"), name="icon_shop_title.png")

    IconTitleAssign = Image(path=sprite_path(r"6408e2c0-d730-4675-9eae-8fd82b20ac96.png"), name="icon_title_assign.png")

    IconTitlePass = Image(path=sprite_path(r"5179ebec-bc81-4b2e-9cfd-2a3354c0d91b.png"), name="icon_title_pass.png")

    BoxApkUpdateDialogTitle = HintBox(x1=26, y1=905, x2=342, y2=967, source_resolution=(720, 1280))

    ButtonAssignmentShortenTime = Image(path=sprite_path(r"1652f06a-5417-49ef-8949-4854772d9ab7.png"), name="工作页面 短缩 时间")

    class Club:
        
        NoteRequestHintBox = HintBox(x1=314, y1=1071, x2=450, y2=1099, source_resolution=(720, 1280))
    
    
        pass
    PointDissmissContestReward = HintPoint(x=604, y=178)

    BoxMissonTabs = HintBox(x1=11, y1=929, x2=703, y2=1030, source_resolution=(720, 1280))

    class CapsuleToys:
        
        NextPageStartPoint = HintPoint(x=360, y=1167)
    
        NextPageEndPoint = HintPoint(x=362, y=267)
    
        IconTitle = Image(path=sprite_path(r"2bd6fe88-99fa-443d-8e42-bb3de5881213.png"), name="日常 扭蛋 页面标题图标")
    
        SliderStartPoint = HintPoint(x=476, y=898)
    
        SliderEndPoint = HintPoint(x=230, y=898)
    
    
        pass
    class SupportCard:
        
        DragDownStartPoint = HintPoint(x=357, y=872)
    
        DragDownEndPoint = HintPoint(x=362, y=194)
    
        TargetSupportCard = HintPoint(x=138, y=432)
    
    
        pass
    TextActivityFundsMax = Image(path=sprite_path(r"a02eba41-6d2f-486e-a07c-2393b927858b.png"), name="text_activity_funds_max.png")

    TextAssignmentCompleted = Image(path=sprite_path(r"70a257e5-aa4d-4fc1-a436-7564d5e83cab.png"), name="text_assignment_completed.png")

    TextContest = Image(path=sprite_path(r"51124743-b57b-48af-b65f-e1704bf7bafa.png"), name="text_contest.png")

    TextContestLastOngoing = Image(path=sprite_path(r"2b466959-f482-4453-b1fa-ebc81a282d30.png"), name="text_contest_last_ongoing.png")

    TextContestNoMemory = Image(path=sprite_path(r"5994b286-e6a4-4c80-ae5b-9d358cac1f11.png"), name="text_contest_no_memory.png")

    TextContestOverallStats = Image(path=sprite_path(r"ff1eb1f1-284c-4f87-be3e-734d0df814b3.png"), name="text_contest_overall_stats.png")

    TextShopPurchased = Image(path=sprite_path(r"d437fd76-9416-4858-b381-6190a82c8fad.png"), name="text_shop_purchased.png")

    TextShopRecommended = Image(path=sprite_path(r"690aaeb1-a725-4f12-bc67-dfb80114d424.png"), name="text_shop_recommended.png")

    TextTabShopAp = Image(path=sprite_path(r"ad28c735-2259-4fdc-9e9c-099c35c7c410.png"), name="text_tab_shop_ap.png")


    pass
class Shop:
    
    ItemLessonNote = Image(path=sprite_path(r"0949c622-9067-4f0d-bac2-3f938a1d2ed2.png"), name="レッスンノート")

    ItemVeteranNote = Image(path=sprite_path(r"b2af59e9-60e3-4d97-8c72-c7ba092113a3.png"), name="ベテランノート")

    ItemSupportEnhancementPt = Image(path=sprite_path(r"835489e2-b29b-426c-b4c9-3bb9f8eb6195.png"), name="サポート強化Pt 支援强化Pt")

    ItemSenseNoteVocal = Image(path=sprite_path(r"c5b7d67e-7260-4f08-a0e9-4d31ce9bbecf.png"), name="センスノート（ボーカル）感性笔记（声乐）")

    ItemSenseNoteDance = Image(path=sprite_path(r"0f7d581d-cea3-4039-9205-732e4cd29293.png"), name="センスノート（ダンス）感性笔记（舞蹈）")

    ItemSenseNoteVisual = Image(path=sprite_path(r"d3cc3323-51af-4882-ae12-49e7384b746d.png"), name="センスノート（ビジュアル）感性笔记（形象）")

    ItemLogicNoteVocal = Image(path=sprite_path(r"a1df3af1-a3e7-4521-a085-e4dc3cd1cc57.png"), name="ロジックノート（ボーカル）理性笔记（声乐）")

    ItemLogicNoteDance = Image(path=sprite_path(r"a9fcaf04-0c1f-4b0f-bb5b-ede9da96180f.png"), name="ロジックノート（ダンス）理性笔记（舞蹈）")

    ItemLogicNoteVisual = Image(path=sprite_path(r"c3f536d6-a04a-4651-b3f9-dd2c22593f7f.png"), name="ロジックノート（ビジュアル）理性笔记（形象）")

    ItemAnomalyNoteVocal = Image(path=sprite_path(r"eef25cf9-afd0-43b1-b9c5-fbd997bd5fe0.png"), name="アノマリーノート（ボーカル）非凡笔记（声乐）")

    ItemAnomalyNoteDance = Image(path=sprite_path(r"df991b42-ed8e-4f2c-bf0c-aa7522f147b6.png"), name="アノマリーノート（ダンス）非凡笔记（舞蹈）")

    ItemAnomalyNoteVisual = Image(path=sprite_path(r"9340b854-025c-40da-9387-385d38433bef.png"), name="アノマリーノート（ビジュアル）非凡笔记（形象）")

    ItemRechallengeTicket = Image(path=sprite_path(r"ea1ba124-9cb3-4427-969a-bacd47e7d920.png"), name="再挑戦チケット 重新挑战券")

    ItemRecordKey = Image(path=sprite_path(r"1926f2f9-4bd7-48eb-9eba-28ec4efb0606.png"), name="記録の鍵  解锁交流的物品")

    class IdolPiece:
        
        倉本千奈_WonderScale = Image(path=sprite_path(r"6720b6e8-ae80-4cc0-a885-518efe12b707.png"), name="倉本千奈 WonderScale 碎片")
    
        篠泽广_光景 = Image(path=sprite_path(r"afa06fdc-a345-4384-b25d-b16540830256.png"), name="篠泽广 光景 碎片")
    
        紫云清夏_TameLieOneStep = Image(path=sprite_path(r"278b7d9c-707e-4392-9677-74574b5cdf42.png"), name="紫云清夏 Tame-Lie-One-Step 碎片")
    
        葛城リーリヤ_白線 = Image(path=sprite_path(r"74ff07b3-d91c-4579-80cd-379ed7020622.png"), name="葛城リーリヤ 白線 碎片")
    
        姫崎薪波_cIclumsy_trick  = Image(path=sprite_path(r"a7f5abf1-982f-4a55-8d41-3ad6f56798e0.png"), name="姫崎薪波 cIclumsy trick 碎片")
    
        花海咲季_FightingMyWay = Image(path=sprite_path(r"2bc00520-0afe-40e5-8743-d33fc6b2945a.png"), name="花海咲季 FightingMyWay 碎片")
    
        藤田ことね_世界一可愛い私 = Image(path=sprite_path(r"135ee57a-d30d-4ba8-83f0-9f1681a49ff7.png"), name="藤田ことね 世界一可愛い私 碎片")
    
        花海佑芽_TheRollingRiceball = Image(path=sprite_path(r"d15959bf-d07b-4f07-948a-c0aeaf17756a.png"), name="花海佑芽 The Rolling Riceball 碎片")
    
        月村手毬_LunaSayMaybe = Image(path=sprite_path(r"868b97a9-492e-4712-b47f-82b97495b019.png"), name="月村手毬 Luna say maybe 碎片")
    
    
        pass

    pass
class Produce:
    
    BoxProduceOngoing = HintBox(x1=179, y1=937, x2=551, y2=1091, source_resolution=(720, 1280))

    ButtonAutoSet = Image(path=sprite_path(r"7ca854b7-34e3-4e31-96eb-d7bb3f11a7d4.png"), name="button_auto_set.png")

    ButtonProduce = Image(path=sprite_path(r"1432980b-5f29-4baa-b73f-b186b82ef9db.png"), name="button_produce.png")

    ButtonProduceStart = Image(path=sprite_path(r"e409f8ec-479d-4cea-b0a8-7eb5bce2f4b2.png"), name="button_produce_start.png")

    ButtonRegular = Image(path=sprite_path(r"90a3b4d4-02c9-45ee-8d39-0f4a608cea3b.png"), name="button_regular.png")

    CheckboxIconNoteBoost = Image(path=sprite_path(r"f34c88b9-769c-4d9f-93c4-ead045233b3a.png"), name="checkbox_icon_note_boost.png")

    CheckboxIconSupportPtBoost = Image(path=sprite_path(r"4d1bf46b-3b4b-487e-885b-8e744cb34931.png"), name="checkbox_icon_support_pt_boost.png")

    BoxModeButtons = HintBox(x1=7, y1=818, x2=713, y2=996, source_resolution=(720, 1280))

    ButtonPIdolOverview = Image(path=sprite_path(r"e88c9ad1-ec37-4fcd-b086-862e1e7ce8fd.png"), name="Pアイドルー覧  P偶像列表展示")

    BoxStepIndicator = HintBox(x1=4, y1=11, x2=405, y2=99, source_resolution=(720, 1280))

    BoxSetCountIndicator = HintBox(x1=17, y1=671, x2=119, y2=707, source_resolution=(720, 1280))

    PointProduceNextSet = HintPoint(x=702, y=832)

    PointProducePrevSet = HintPoint(x=14, y=832)

    IconPIdolLevel = Image(path=sprite_path(r"30a6f399-6999-4f04-bb77-651e0214112f.png"), name="P偶像卡上的等级图标")

    KbIdolOverviewName = HintBox(x1=140, y1=16, x2=615, y2=97, source_resolution=(720, 1280))

    ButtonResume = Image(path=sprite_path(r"ccbcb114-7f73-43d1-904a-3a7ae660c531.png"), name="再開する")

    BoxResumeDialogProduceType = HintBox(x1=186, y1=491, x2=541, y2=550, source_resolution=(720, 1280))

    BoxResumeDialogIdolTitle = HintBox(x1=194, y1=794, x2=523, y2=840, source_resolution=(720, 1280))

    RadioTextSkipCommu = Image(path=sprite_path(r"8626458a-63e7-4288-96e3-cbd624e64b75.png"), name="radio_text_skip_commu.png")

    TextAPInsufficient = Image(path=sprite_path(r"4883c564-f950-4a29-9f5f-6f924123cd22.png"), name="培育 AP 不足提示弹窗 标题")

    ButtonRefillAP = Image(path=sprite_path(r"eaba6ebe-f0df-4918-aee5-ef4e3ffedcf0.png"), name="确认恢复AP按钮")

    ScreenshotNoEnoughAp2 = Image(path=sprite_path(r"8ab34547-96bd-459b-8d3d-a8bd38ff010d.png"), name="screenshot_no_enough_ap_2.png")

    ButtonSkipLive = Image(path=sprite_path(r"e5e84f9e-28da-4cf4-bcba-c9145fe39b07.png"), name="培育结束跳过演出按钮")

    TextSkipLiveDialogTitle = Image(path=sprite_path(r"b6b94f21-ef4b-4425-9c7e-ca2b574b0add.png"), name="跳过演出确认对话框标题")

    TextRentAvailable = Image(path=sprite_path(r"9d89731a-3e5d-4e1c-9c25-dbca52f8ada6.png"), name="text_rent_available.png")


    pass
class InPurodyuusu:
    
    A = Image(path=sprite_path(r"90c8fe73-a906-43c2-860d-f608c29fe9f1.png"), name="A.png")

    AcquireBtnDisabled = Image(path=sprite_path(r"c1e8886f-f3fb-4b05-80f3-2014bc403554.png"), name="acquire_btn_disabled.png")

    ButtonCancel = Image(path=sprite_path(r"349c643a-04cc-4e5c-a1d2-582e14f0a368.png"), name="button_cancel.png")

    ButtonComplete = Image(path=sprite_path(r"9f7b489c-d68e-46ee-82d3-8bed333a1efa.png"), name="button_complete.png")

    ButtonFinalPracticeDance = Image(path=sprite_path(r"9575ec77-9825-40d9-8803-a311843a372b.png"), name="button_final_practice_dance.png")

    ButtonFinalPracticeVisual = Image(path=sprite_path(r"976befa5-b285-45be-b04b-d3c0386f4699.png"), name="button_final_practice_visual.png")

    ButtonFinalPracticeVocal = Image(path=sprite_path(r"34f01361-adf1-4aab-8f2b-697a3c3bd17f.png"), name="button_final_practice_vocal.png")

    ButtonFollowNoIcon = Image(path=sprite_path(r"6dcd939f-e0b2-4346-888e-d92ad0fac266.png"), name="button_follow_no_icon.png")

    ButtonIconStudy = Image(path=sprite_path(r"0dbdce25-b2e6-44fa-b0a4-b82467a94ba7.png"), name="button_icon_study.png")

    ButtonIconStudyVisual = Image(path=sprite_path(r"0ef5eaa1-be89-4b70-a04c-5be3d7499c68.png"), name="button_icon_study_visual.png")

    ButtonLeave = Image(path=sprite_path(r"1e570f83-77dd-40b8-b2e5-516c09f8faa9.png"), name="button_leave.png")

    ButtonNextNoIcon = Image(path=sprite_path(r"db140c0c-03d2-4e34-b8cf-a0c1c72d24ab.png"), name="button_next_no_icon.png")

    ButtonNotAcquire = Image(path=sprite_path(r"1634e858-6312-4c55-9734-4ecde427ba19.png"), name="button_not_acquire.png")

    ButtonRetry = Image(path=sprite_path(r"c1af1812-63d1-439d-b129-fca2106b5a62.png"), name="button_retry.png")

    ButtonTextActionOuting = Image(path=sprite_path(r"dfacb774-b6d8-45b5-9550-367fd3eeee19.png"), name="button_text_action_outing.png")

    ButtonTextAllowance = Image(path=sprite_path(r"4e0c6e03-262f-4c9e-8b5a-d00439376672.png"), name="button_text_allowance.png")

    ButtonTextConsult = Image(path=sprite_path(r"466ac973-10ec-4676-9f6f-1e2cd1fe97b4.png"), name="button_text_consult.png")

    IconClearBlue = Image(path=sprite_path(r"bee2adba-e36d-4c79-9061-be815164951a.png"), name="icon_clear_blue.png")

    IconTitleAllowance = Image(path=sprite_path(r"17568434-4ad5-46c5-b142-1b1f05614c08.png"), name="icon_title_allowance.png")

    IconTitleStudy = Image(path=sprite_path(r"284276f2-67b3-4d32-8358-516e136f6af6.png"), name="icon_title_study.png")

    LootboxSliverLock = Image(path=sprite_path(r"d18bf286-a85c-4866-83d5-3b3fa2e66fe3.png"), name="lootbox_sliver_lock.png")

    LootBoxSkillCard = Image(path=sprite_path(r"43f318d7-2f15-4b6f-863b-e2261bc97bbb.png"), name="loot_box_skill_card.png")

    M = Image(path=sprite_path(r"2c141963-1bc6-4c4f-9185-68a7d841d040.png"), name="M.png")

    BoxWeeksUntilExam = HintBox(x1=11, y1=8, x2=237, y2=196, source_resolution=(720, 1280))

    TextActionVocal = Image(path=sprite_path(r"d6b64759-26b7-45b1-bf8e-5c0d98611e0d.png"), name="Vo. レッスン")

    TextActionDance = Image(path=sprite_path(r"303cccc1-c674-4d3a-8c89-19ea729fdbef.png"), name="Da. レッスン")

    TextActionVisual = Image(path=sprite_path(r"cc8a495d-330d-447d-8a80-a8a6ecc409c5.png"), name="Vi. レッスン")

    BoxExamTop = HintBox(x1=5, y1=2, x2=712, y2=55, source_resolution=(720, 1280))

    PDrinkIcon = Image(path=sprite_path(r"ee84cfe3-f36d-42c4-a27c-033e8afeb077.png"), name="p_drink_icon.png")

    PItemIconColorful = Image(path=sprite_path(r"3eb3edc8-be33-4fcd-866a-0ae6fbbb932c.png"), name="p_item_icon_colorful.png")

    PSkillCardIconBlue = Image(path=sprite_path(r"ba402073-ef1b-42d0-8cfb-84ffefdc0c43.png"), name="p_skill_card_icon_blue.png")

    PSkillCardIconColorful = Image(path=sprite_path(r"a35bdd08-b319-47d0-be24-26dcf9789a04.png"), name="p_skill_card_icon_colorful.png")

    Rest = Image(path=sprite_path(r"f24a70f9-304e-4142-8002-be6baf523206.png"), name="rest.png")

    RestConfirmBtn = Image(path=sprite_path(r"2c750883-32c9-451b-817e-2fc1a190020c.png"), name="rest_confirm_btn.png")

    ButtonIconOuting = Image(path=sprite_path(r"8ded6c98-85ea-4858-a66d-4fc8caecb7c5.png"), name="行动按钮图标 外出（おでかけ）")

    TextGoalClearNext = Image(path=sprite_path(r"05890a1b-8764-4e9f-9d21-65d292c22e13.png"), name="培育目标达成 NEXT 文字")

    BoxLessonCards5_1 = HintBox(x1=16, y1=882, x2=208, y2=1136, source_resolution=(720, 1280))

    BoxNoSkillCard = HintBox(x1=180, y1=977, x2=529, y2=1026, source_resolution=(720, 1280))

    TitleIconOuting = Image(path=sprite_path(r"ee4e512b-4982-49b6-9c71-31984b58e1d0.png"), name="外出（おでかけ）页面 标题图标")

    TextPDrinkMaxConfirmTitle = Image(path=sprite_path(r"582d36c0-0916-4706-9833-4fbc026701f5.png"), name="P饮料溢出 不领取弹窗标题")

    IconTitleSkillCardRemoval = Image(path=sprite_path(r"bab6c393-692c-4681-ac0d-76c0d9dabea6.png"), name="技能卡自选删除 标题图标")

    ButtonRemove = Image(path=sprite_path(r"00551158-fee9-483f-b034-549139a96f58.png"), name="削除")

    TextPDrink = Image(path=sprite_path(r"8c179a21-be6f-4db8-a9b0-9afeb5c36b1c.png"), name="文本「Pドリンク」")

    BoxSelectPStuffComfirm = HintBox(x1=256, y1=1064, x2=478, y2=1128, source_resolution=(720, 1280))

    TextClaim = Image(path=sprite_path(r"c948f136-416f-447e-8152-54a1cd1d1329.png"), name="文字「受け取る」")

    TextPItem = Image(path=sprite_path(r"0c0627be-4a09-4450-a078-1858d3ace532.png"), name="文字「Pアイテム」")

    TextSkillCard = Image(path=sprite_path(r"d271a24f-efe8-424d-8fd5-f6b3756ba4ca.png"), name="文字「スキルカード」")

    BoxSkillCardAcquired = HintBox(x1=194, y1=712, x2=528, y2=765, source_resolution=(720, 1280))

    IconSkillCardEventBubble = Image(path=sprite_path(r"6b58d90d-2e5e-4b7f-bc01-941f2633de89.png"), name="技能卡事件气泡框图标")

    ScreenshotSkillCardEnhanceDialog = Image(path=sprite_path(r"249bb7b2-68cb-44b0-840a-855f76405177.png"), name="screenshot_skill_card_enhance_dialog.png")

    IconTitleSkillCardEnhance = Image(path=sprite_path(r"79abd239-5eed-4195-9fa8-d729daa874ca.png"), name="技能卡强化 标题 图标")

    ButtonEnhance = Image(path=sprite_path(r"da439e8c-3b74-4371-9657-0736d826c7d1.png"), name="技能卡 强化按钮")

    T = Image(path=sprite_path(r"16fbc93d-b294-4001-b4e9-ee2af181415f.png"), name="睡意卡字母 T（眠気）")

    IconSp = Image(path=sprite_path(r"d982d2b5-4bc0-4ae9-a516-f29c2848d64b.png"), name="SP 课程图标")

    BoxCommuEventButtonsArea = HintBox(x1=14, y1=412, x2=703, y2=1089, source_resolution=(720, 1280))

    TextSelfStudyVocal = Image(path=sprite_path(r"c78c38cc-7b61-4dc4-820d-0a5b684ef52e.png"), name="文化课事件 自习 声乐")

    TextSelfStudyDance = Image(path=sprite_path(r"83d0a033-466c-463a-bb8c-be0f2953e9b2.png"), name="文化课事件 自习 舞蹈")

    TextSelfStudyVisual = Image(path=sprite_path(r"4695f96b-c4f5-4bb6-a021-a13b6ceb2883.png"), name="文化课事件 自习 形象")

    TextAsariProduceEnd = Image(path=sprite_path(r"15ab4cd6-00ef-4fc2-b8b2-3f0fea8d121c.png"), name="text_asari_produce_end.png")

    TextButtonExamSkipTurn = Image(path=sprite_path(r"58d8977e-626a-4122-834e-be322e504ce8.png"), name="text_button_exam_skip_turn.png")

    TextClearUntil = Image(path=sprite_path(r"e4ed1a58-a1d3-406c-8433-9cb8a116870b.png"), name="text_clear_until.png")

    TextDance = Image(path=sprite_path(r"1fb62eeb-c8b5-44aa-a6d9-52e2efdeee7f.png"), name="text_dance.png")

    TextFinalProduceRating = Image(path=sprite_path(r"46708b7b-e2a5-4247-9050-933304c3e7d4.png"), name="text_final_produce_rating.png")

    TextOneWeekRemaining = Image(path=sprite_path(r"743d1b3b-a219-4240-89da-5dfd06330699.png"), name="text_one_week_remaining.png")

    TextPerfectUntil = Image(path=sprite_path(r"a6ee9746-3953-4867-babb-8397697cdcef.png"), name="text_perfect_until.png")

    TextPleaseSelectPDrink = Image(path=sprite_path(r"f625c4b0-1e06-4fd0-b047-080b2760e43e.png"), name="text_please_select_p_drink.png")

    TextPDiary = Image(path=sprite_path(r"e48a9693-13cd-463d-b4dc-d8cf63536c33.png"), name="text_p_diary.png")

    TextPDrinkMax = Image(path=sprite_path(r"0b15b72e-d3f0-414c-8c9b-3452e85d8a04.png"), name="text_p_drink_max.png")

    TextSenseiTipConsult = Image(path=sprite_path(r"a7780df5-74f7-420f-b845-0442454832df.png"), name="text_sensei_tip_consult.png")

    TextSenseiTipDance = Image(path=sprite_path(r"40ea9658-fcf6-44f9-b4f7-b5183f0c7089.png"), name="text_sensei_tip_dance.png")

    TextSenseiTipRest = Image(path=sprite_path(r"4055f351-9a78-4a01-bba7-081b380b0ef3.png"), name="text_sensei_tip_rest.png")

    TextSenseiTipVisual = Image(path=sprite_path(r"0cc78ac2-db49-4827-a258-63c9963201fe.png"), name="text_sensei_tip_visual.png")

    TextSenseiTipVocal = Image(path=sprite_path(r"175364b3-8f02-4c79-ae1c-cf1feb2908e9.png"), name="text_sensei_tip_vocal.png")

    TextSkipTurnDialog = Image(path=sprite_path(r"6265f543-48f4-43f3-9e80-f8bb184e3d57.png"), name="text_skip_turn_dialog.png")

    TextVisual = Image(path=sprite_path(r"f27bdd8b-6287-4278-b82d-3a9f6f0ef427.png"), name="text_visual.png")

    class Action:
        
        ActionStudy = Image(path=sprite_path(r"b29ebe51-1a79-4c4d-b6f5-9583602c0595.png"), name="action_study.png")
    
        PDorinkuBg = Image(path=sprite_path(r"26c68c62-8001-4238-95a9-b84faa735d65.png"), name="p_dorinku_bg.png")
    
        PDorinkuBgMask = Image(path=sprite_path(r"2e076e6e-0ed7-4f85-93ae-52bb07155ca1.png"), name="p_dorinku_bg_mask.png")
    
        PItem = Image(path=sprite_path(r"21bc6029-9370-4ffe-a0e8-120b899173a1.png"), name="p_item.png")
    
        VocalWhiteBg = Image(path=sprite_path(r"1ee95dc4-cfef-40df-b86e-534113bec38a.png"), name="vocal_white_bg.png")
    
    
        pass

    pass
class Kuyo:
    
    ButtonStartGame = Image(path=sprite_path(r"c5956d09-9154-4360-a280-443e8000cb85.png"), name="button_start_game.png")

    ButtonTab3Speedup = Image(path=sprite_path(r"0aebab91-020b-4bbd-baeb-920579a24531.png"), name="button_tab3_speedup.png")


    pass